package cz.muni.fi.pv168.graves;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This is example of an JDBC implementation of some methods.
 *
 *
 * To create a table for graves:
 *
 * CREATE TABLE GRAVE
 *   (ID INTEGER NOT NULL PRIMARY KEY GENERATED ALWAYS AS IDENTITY,
 *   GRAVECOLUMN INTEGER,
 *   GRAVEROW INTEGER,
 *   CAPACITY INTEGER,
 *   NOTE VARCHAR(50)
 * );
 *
 * JDBC (derby.jar, derbyclient.jar, derbynet.jar) Driver
 * must be available on the classpath.
 * Can be downloaded from:
 * http://db.apache.org/derby/releases/release-10.7.1.1.cgi
 * To add: Libraries ->right click ->Add JAR/Folder ->select these 3 jars
 *
 * @author Eva Kucirkova (eva.k@mail.muni.cz)
 */
public class CemeteryManagerImpl implements CemeteryManager {

    private Connection conn;
    private static final Logger logger = Logger.getLogger(
            CemeteryManagerImpl.class.getName());

    public CemeteryManagerImpl() {
        try {
            conn = DriverManager.getConnection("jdbc:derby://localhost:1527/graveyard-test", "death", "death");
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "Error when connecting to DB", ex);
            throw new RuntimeException("Error when connecting to DB", ex);
        }
    }

    public Collection<Grave> getAllGraves() {
        PreparedStatement st = null;
        try {
            st = conn.prepareStatement("SELECT * from grave");
            ResultSet rs = st.executeQuery();
            Collection<Grave> found = new HashSet<Grave>();
            while (rs.next()) {
                Grave result = new Grave();
                result.setId(rs.getLong("id"));
                result.setCapacity(rs.getInt("capacity"));
                result.setColumn(rs.getInt("gravecolumn"));
                result.setRow(rs.getInt("graverow"));
                result.setNote(rs.getString("note"));
                found.add(result);
            }
            return Collections.unmodifiableCollection(found);
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "Error when getting all graves from DB", ex);
            throw new RuntimeException("Error when getting all graves from DB", ex);
        } finally {
            if (st != null) {
                try {
                    st.close();
                } catch (SQLException ex) {
                    logger.log(Level.SEVERE, "Error when closing connection", ex);
                }
            }
        }


    }

    public Grave getGraveById(Long id) {
        if (id == null) {
            throw new IllegalArgumentException("id");
        }
        PreparedStatement st = null;
        try {
            st = conn.prepareStatement(
                    "SELECT id, graverow,gravecolumn,capacity,note FROM GRAVE WHERE ID = ?");
            st.setLong(1, id);

            ResultSet rs = st.executeQuery();

            if (rs.next()) {
                Grave result = new Grave();
                result.setId(rs.getLong("id"));
                result.setCapacity(rs.getInt("capacity"));
                result.setColumn(rs.getInt("gravecolumn"));
                result.setRow(rs.getInt("graverow"));
                result.setNote(rs.getString("note"));
                assert !rs.next();
                return result;
            } else {
                return null;
            }

        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "Error when getting grave from DB", ex);
            throw new RuntimeException("Error when getting grave from DB", ex);
        } finally {
            if (st != null) {
                try {
                    st.close();
                } catch (SQLException ex) {
                    logger.log(Level.SEVERE, "Error when closing connection", ex);
                }
            }
        }
    }

    public void addGrave(Grave grave) {
        if (grave == null) {
            throw new IllegalArgumentException("grave");
        }
        if (grave.getId() != null) {
            throw new IllegalArgumentException("grave id is already set");
        }
        if (grave.getCapacity() <= 0) {
            throw new IllegalArgumentException("grave capacity is negative");
        }

        PreparedStatement st = null;
        try {
            st = conn.prepareStatement(
                    "INSERT INTO GRAVE (capacity,graverow,gravecolumn,note) VALUES (?,?,?,?)",
                    Statement.RETURN_GENERATED_KEYS);
            st.setInt(1, grave.getCapacity());
            st.setInt(2, grave.getRow());
            st.setInt(3, grave.getColumn());
            st.setString(4, grave.getNote());

            int count = st.executeUpdate();
            assert count == 1;

            Long id = getId(st.getGeneratedKeys());
            grave.setId(id);

        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "Error when inserting grave into DB", ex);
            throw new RuntimeException("Error when inserting into DB", ex);
        } finally {
            if (st != null) {
                try {
                    st.close();
                } catch (SQLException ex) {
                    logger.log(Level.SEVERE, "Error when closing connection", ex);
                }
            }
        }

    }

    public void deleteGrave(Grave grave) {
        if (grave == null) {
            throw new IllegalArgumentException("grave");
        }
        if (grave.getBodies() != null && !grave.getBodies().isEmpty()) {
            throw new IllegalArgumentException("can't delete non-empty grave");
        }
        PreparedStatement st = null;
        try {
            st = conn.prepareStatement("DELETE FROM grave WHERE id=?");
            st.setLong(1, grave.getId());
            if (st.executeUpdate() == 0) {
                throw new IllegalArgumentException("grave not found");
            }
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "Error when deleting grave from DB", ex);
            throw new RuntimeException("Error when deleting grave from DB", ex);
        } finally {
            if (st != null) {
                try {
                    st.close();
                } catch (SQLException ex) {
                    logger.log(Level.SEVERE, "Error when closing connection", ex);
                }
            }
        }


    }

    public void updateGrave(Grave grave) {
        throw new UnsupportedOperationException("method not implemented yet");
    }

    private static Long getId(ResultSet keys) throws SQLException {
        if (keys.getMetaData().getColumnCount() != 1) {
            throw new IllegalArgumentException("Given ResultSet contains more columns");
        }
        if (keys.next()) {
            Long result = keys.getLong(1);
            if (keys.next()) {
                throw new IllegalArgumentException("Given ResultSet contains more rows");
            }
            return result;
        } else {
            throw new IllegalArgumentException("Given ResultSet contain no rows");
        }
    }
}
